import Schema from './schema';
import { boolean as locale } from './locale';
import isAbsent from './util/isAbsent';
export function create() {
  return new BooleanSchema();
}
export default class BooleanSchema extends Schema {
  constructor() {
    super({
      type: 'boolean',

      check(v) {
        if (v instanceof Boolean) v = v.valueOf();
        return typeof v === 'boolean';
      }

    });
    this.withMutation(() => {
      this.transform((value, _raw, ctx) => {
        if (ctx.spec.coarce && !ctx.isType(value)) {
          if (/^(true|1)$/i.test(String(value))) return true;
          if (/^(false|0)$/i.test(String(value))) return false;
        }

        return value;
      });
    });
  }

  isTrue(message = locale.isValue) {
    return this.test({
      message,
      name: 'is-value',
      exclusive: true,
      params: {
        value: 'true'
      },

      test(value) {
        return isAbsent(value) || value === true;
      }

    });
  }

  isFalse(message = locale.isValue) {
    return this.test({
      message,
      name: 'is-value',
      exclusive: true,
      params: {
        value: 'false'
      },

      test(value) {
        return isAbsent(value) || value === false;
      }

    });
  }

  default(def) {
    return super.default(def);
  } // concat<TOther extends BooleanSchema<any, any>>(schema: TOther): TOther;


  defined(msg) {
    return super.defined(msg);
  }

  optional() {
    return super.defined();
  }

  required(msg) {
    return super.required(msg);
  }

  notRequired() {
    return super.notRequired();
  }

  nullable() {
    return super.nullable();
  }

  nonNullable(msg) {
    return super.nonNullable(msg);
  }

  strip(v) {
    return super.strip(v);
  }

}
create.prototype = BooleanSchema.prototype;